@@ -84,9 +84,26 @@ sdf_read_column <- function(x, column) {
   column
 }
 
+#' Collect a Spark DataFrame into R.
+#'
+#' Collects a Spark dataframe into R.
+#'
+#' @param object Spark dataframe to collect
+#' @param ... Additional options.
+#'
+#' @export
+sdf_collect <- function(object, ...) {
+  sc <- spark_connection(object)
+
+  if (sdf_is_streaming(object))
+    sdf_collect_stream(object, ...)
+  else
+    sdf_collect_static(object, ...)
+}
+
 # Read a Spark Dataset into R.
 #' @importFrom dplyr as_data_frame
-sdf_collect <- function(object) {
+sdf_collect_static <- function(object, ...) {
   sc <- spark_connection(object)
   sdf <- spark_dataframe(object)
 
@@ -164,6 +181,28 @@ sdf_split <- function(object,
 #'   a length-one character vector, giving the name of a Spark aggregation function
 #'   to be called; a named \R list mapping column names to an aggregation method,
 #'   or an \R function that is invoked on the grouped dataset.
+#'
+#'@examples
+#'\dontrun{
+#' library(sparklyr)
+#' library(dplyr)
+#'
+#' sc <- spark_connect(master = "local")
+#' iris_tbl <- sdf_copy_to(sc, iris, name = "iris_tbl", overwrite = TRUE)
+#'
+#' # aggregating by mean
+#' iris_tbl %>%
+#'   mutate(Petal_Width = ifelse(Petal_Width > 1.5, "High", "Low" )) %>%
+#'   sdf_pivot(Petal_Width ~ Species,
+#'             fun.aggregate = list(Petal_Length = "mean"))
+#'
+#' # aggregating all observations in a list
+#' iris_tbl %>%
+#'   mutate(Petal_Width = ifelse(Petal_Width > 1.5, "High", "Low" )) %>%
+#'   sdf_pivot(Petal_Width ~ Species,
+#'             fun.aggregate = list(Petal_Length = "collect_list"))
+#'}
+#'
 #' @export
 sdf_pivot <- function(x, formula, fun.aggregate = "count") {
   sdf <- spark_dataframe(x)
@@ -234,13 +273,13 @@ sdf_separate_column <- function(x,
                                 column,
                                 into = NULL)
 {
-  column <- ensure_scalar_character(column)
+  column <- cast_string(column)
 
   # extract spark dataframe reference, connection
   sdf <- spark_dataframe(x)
   sc <- spark_connection(x)
 
-  into_is_set <- ensure_scalar_boolean(!is.null(into))
+  into_is_set <- cast_scalar_logical(!is.null(into))
 
   # when 'into' is NULL, we auto-generate a names -> index map
   if (is.null(into)) {
