@@ -80,7 +80,9 @@ spark_version_from_home <- function(spark_home, default = NULL) {
       candidateVersions <- list(
         list(path = "lib", pattern = "spark-assembly-([0-9\\.]*)-hadoop.[0-9\\.]*\\.jar"),
         list(path = "yarn", pattern = "spark-([0-9\\.]*)-preview-yarn-shuffle\\.jar"),
-        list(path = "yarn", pattern = "spark-([0-9\\.]*)-yarn-shuffle\\.jar")
+        list(path = "yarn", pattern = "spark-([0-9\\.]*)-yarn-shuffle\\.jar"),
+        list(path = "lib", pattern = "spark-([0-9\\.]*)-preview-yarn-shuffle\\.jar"),
+        list(path = "lib", pattern = "spark-([0-9\\.]*)-yarn-shuffle\\.jar")
       )
 
       candidateFiles <- lapply(candidateVersions, function(e) {
@@ -88,7 +90,8 @@ spark_version_from_home <- function(spark_home, default = NULL) {
           list(
             files = list.files(
               file.path(spark_home, e$path),
-              pattern = e$pattern
+              pattern = e$pattern,
+              recursive = TRUE
             )
           )
         )
