@@ -1,8 +1,19 @@
 context("copy data")
+
 sc <- testthat_spark_connection()
 
+test_that("sdf_copy_to works for default serializer", {
+  df <- matrix(0, ncol = 5, nrow = 2) %>% dplyr::as_data_frame()
+  df_tbl <- sdf_copy_to(sc, df, overwrite = TRUE)
+
+  expect_equal(
+    sdf_nrow(df_tbl),
+    2
+  )
+})
+
 test_that("sdf_copy_to works for scala serializer", {
-  skip_on_cran()
+  skip_livy()
 
   df <- matrix(0, ncol = 5, nrow = 2) %>% dplyr::as_data_frame()
   df_tbl <- sdf_copy_to(sc, df, overwrite = TRUE, serializer = "csv_file_scala")
@@ -14,7 +25,7 @@ test_that("sdf_copy_to works for scala serializer", {
 })
 
 test_that("sdf_copy_to works for csv serializer", {
-  skip_on_cran()
+  skip_livy()
 
   df <- matrix(0, ncol = 5, nrow = 2) %>% dplyr::as_data_frame()
   df_tbl <- sdf_copy_to(sc, df, overwrite = TRUE, serializer = "csv_file")
